/*
 * @plotColor.java Version 1.1 05/20/2002
 *
 * Copyright (c) 2002 Kansas Geological Survey
 * 1930 Constant Avenue, Lawrence, Kansas, 66047, U.S.A.
 * All Rights Reserved.
 */

package plot;

import java.awt.*;

/** Class plotColor
 *  <p> This method will allow the user to display their plots in different
 *      shading schemes
 *
 *  @version 1.1 05/20/2002
 *  @author  John R. Victorine
 */

public class plotColor extends Canvas
{
  private static final int MAXCOLORS      = 768;

  // Black to White Combinations

  public  static final int RED_BLUE_GREEN = 0; // BLACK-GREEN-CYAN-WHITE
  public  static final int RED_GREEN_BLUE = 1; // BLACK-BLUE-CYAN-WHITE
  public  static final int GREEN_RED_BLUE = 2; // BLACK-BLUE-VIOLET-WHITE
  public  static final int GREEN_BLUE_RED = 3; // BLACK-RED-VIOLET-WHITE
  public  static final int BLUE_RED_GREEN = 4; // BLACK-GREEN-YELLOW-WHITE
  public  static final int BLUE_GREEN_RED = 5; // BLACK-RED-YELLOW-WHITE

  private static final int FORWARD        = 0; // Original Color schema
  private static final int BACKWARD       = 1; // Reverse Color Schema

  private              int    iType  = 0;  // 0 = Automatic; 1 = Manual Colors

  private              int    iColor = BLUE_GREEN_RED;
  private              double dMinimum = 0.0;
  private              double dMaximum = 0.0;

  private              int    iRed   = 255;
  private              int    iBlue  = 255;
  private              int    iGreen = 255;

  private              int    iColor1 = 255;
  private              int    iColor2 = 255;
  private              int    iColor3 = 255;

  private              int    iXBegin = 0;
  private              int    iYBegin = 0;
  private              int    iWidth  = 0;
  private              int    iHeight = 0;

  private              double dLower[]  = { 80.0, 60.0, 40.0, 20.0,  0.0};
  private              double dUpper[]  = {100.0, 80.0, 60.0, 40.0, 20.0};
  private              int    iRGB[][]  = {{255,0,0},        // Red
                                           {255,255,0},      // Yellow
                                           {0,255,0},        // Green
                                           {0,0,255},        // Blue
                                           {255,0,255}};     // Violet
  private              int    iRangeU[]  = {768, 614, 461, 307, 154};
  private              int    iRangeL[]  = {614, 461, 307, 154, 0};

  /** Constructor plotColor()      NO PLOT SCALE IS CREATED
   *  <p> This is the Constructor for this class using the BLACK-BLUE-CYAN-WHITE
   *      Combination colors.
   */

  public plotColor()
  {
    iColor   = BLUE_GREEN_RED;
    dMinimum = 0.0;
    dMaximum = 100.0;
  }

  /** Constructor plotColor()      NO PLOT SCALE IS CREATED
   *  <p> This is the Constructor for this class using the BLACK-BLUE-CYAN-WHITE
   *      Combination colors.
   *  @param minimum = The minimum data value
   *  @param maximum = The maximum data value
   */

  public plotColor(double minimum, double maximum)
  {
    iColor   = BLUE_GREEN_RED;
    dMinimum = minimum;
    dMaximum = maximum;
  }

  /** Constructor plotColor()      NO PLOT SCALE IS CREATED
   *  <p> This is the Constructor for this class.
   *  @param color   = Color Shading ID
   *  @param minimum = The minimum data value
   *  @param maximum = The maximum data value
   */

  public plotColor(int color, double minimum, double maximum)
  {
    iColor   = color;
    dMinimum = minimum;
    dMaximum = maximum;
  }

  /** Constructor plotColor()      PLOT SCALE IS CREATED
   *  <p> This is the Constructor for this class.
   *  @param color   = Color Shading ID
   *  @param minimum = The minimum data value
   *  @param maximum = The maximum data value
   *  @param xStart  = The X position of upper left point
   *  @param yStart  = The Y position of upper left point
   *  @param width   = The Width of Scale.
   *  @param height  = The Height of Scale.
   */

  public plotColor(int color, double minimum, double maximum,
                   int xStart, int yStart, int width, int height)
  {
    iColor   = color;
    dMinimum = minimum;
    dMaximum = maximum;

    iXBegin = xStart;
    iYBegin = yStart;
    iWidth  = width;
    iHeight = height;
  }

  /** Constructor plotColor()      PLOT SCALE IS CREATED
   *  <p> This is the Constructor for this class.
   *  @param color   = Color Shading ID
   *  @param xStart  = The X position of upper left point
   *  @param yStart  = The Y position of upper left point
   *  @param width   = The Width of Scale.
   *  @param height  = The Height of Scale.
   */

  public plotColor(int color, int xStart, int yStart, int width, int height)
  {
    iColor   = color;
    dMinimum = 0.0;
    dMaximum = 100.0;

    iXBegin = xStart;
    iYBegin = yStart;
    iWidth  = width;
    iHeight = height;
  }

  /** Method close()
   * <p> This method will set the global variables to null in order to force
   *     JAVA to free the memory allocated for this panel.
   */

  public void close()
  {
    dLower  = null;
    dUpper  = null;
    iRGB    = null;
    iRangeU = null;
    iRangeL = null;
  }

  /** Method convertDataToColor()
   *  <p> This method will convert the data type to a specific color value from
   *      0 to the Maximum Color Value 768.
   *  @param data = The data value to be converted.
   */

  public int convertDataToColor(double data)
  {
    double delta = (dMaximum -  dMinimum) / (double) MAXCOLORS;
    return ( MAXCOLORS - (int)((dMaximum - data) / delta) );
  }

  /** Method computeColor()
   *  <p> This method will compute the color value based on the Color ID
   *      Selected.
   *  @param iValue = the Value that is being converted to a color
   */

  public void computeColor(int iValue)
  {
    if (iType == 0)
    {
      if (iValue < 256)
      {
        iColor1 = 0;
        iColor2 = 0;
        iColor3 = iValue;
      }
      else if (iValue < 512)
      {
        iColor1 = 0;
        iColor2 = iValue - 256;
        iColor3 = 255;
      }
      else if (iValue < 768)
      {
        iColor1 = iValue - 512;
        iColor2 = 255;
        iColor3 = 255;
      }
      else
      {
        iColor1 = 255;
        iColor2 = 255;
        iColor3 = 255;
      }

      switch (iColor)
      {
        case RED_BLUE_GREEN: // BLACK-RED-VIOLET-WHITE
          iRed   = iColor1;
          iGreen = iColor3;
          iBlue  = iColor2;
          break;

        case RED_GREEN_BLUE: // BLACK-RED-YELLOW-WHITE
          iRed   = iColor1;
          iGreen = iColor2;
          iBlue  = iColor3;
          break;

        case GREEN_RED_BLUE: // BLACK-GREEN-YELLOW-WHITE
          iRed   = iColor2;
          iGreen = iColor1;
          iBlue  = iColor3;
          break;

        case GREEN_BLUE_RED: // BLACK-GREEN-CYAN-WHITE
          iRed   = iColor3;
          iGreen = iColor1;
          iBlue  = iColor2;
          break;

        case BLUE_RED_GREEN: // BLACK-BLUE-VIOLET-WHITE
          iRed   = iColor2;
          iGreen = iColor3;
          iBlue  = iColor1;
          break;

        case BLUE_GREEN_RED: // BLACK-BLUE-CYAN-WHITE
          iRed   = iColor3;
          iGreen = iColor2;
          iBlue  = iColor1;
          break;
      }
    }
    else
    {
      iValue = 768 - iValue;

      if (iValue <= iRangeL[4])
      {
        iRed   = 255;
        iGreen = 255;
        iBlue  = 255;
      }
      else if (iValue > iRangeU[0])
      {
        iRed   = 0;
        iGreen = 0;
        iBlue  = 0;
      }

      for (int i=0; i<5; i++)
      {
        if ((iValue > iRangeL[i]) && (iValue < iRangeU[i]))
        {
          iRed   = iRGB[i][0];
          iGreen = iRGB[i][1];
          iBlue  = iRGB[i][2];
        }
      }
    }
  }

  /** Method getRed()
   *  <p> This method will return the red value of the color
   *  @return iRed = The Red int value
   */

  public int getRed() { return (iRed); }

  /** Method getBlue()
   *  <p> This method will return the red value of the color
   *  @return iBlue = The Blue int value
   */

  public int getBlue() { return (iBlue); }

  /** Method getGreen()
   *  <p> This method will return the red value of the color
   *  @return iGreen = The Green int value
   */

  public int getGreen() { return (iGreen); }

  /** Method setColor()
   *  <p> This method will change the set the color value
   *  @param color   = Color Shading ID
   */

  public void setColor(int color) { iColor = color; }

  /** Constructor setMinMaxValue()
   *  <p> This method will set the Minimum and Maximum Value.
   *  @param minimum = The minimum data value
   *  @param maximum = The maximum data value
   */

  public void setMinMaxValue(double minimum, double maximum)
  {
    dMinimum = minimum;
    dMaximum = maximum;
  }

  /** Method setType()
   *  <p> This method will set the Type of Plot to produce.
   *  @param type = Type of Plot 0 = Automatic, 1 = Manaul
   */

  public void setType(int type) { iType = type; }

  /** Method setManualData()
   *  <p> This method will set the Manual Data Arrays
   *  @param lower = lower grid values
   *  @param upper = upper grid values
   *  @pram  rgb   = RGB Grid Values
   */

  public void setManualData(double lower[], double upper[], int rgb[][])
  {
    int    i = 0;

    for (i=0; i<5; i++)
    {
      dLower[i] = lower[i];
      dUpper[i] = upper[i];

      iRangeU[i] = convertDataToColor(dUpper[i]);
      iRangeL[i] = convertDataToColor(dLower[i]);

      iRGB[i][0] = rgb[i][0];
      iRGB[i][1] = rgb[i][1];
      iRGB[i][2] = rgb[i][2];
    }
  }

  /** Method drawScale()
   *  <p> This method will paint the scale bar
   *  @param g - Graphics Handle;
   */

  public void drawScale(Graphics g)
  {
    int    i      = 0;
    int    iCount = 0;
    int    iValue = -1;
    int    iRed   = -1;
    int    iGreen = -1;
    int    iBlue  = -1;

    double dMin = Math.ceil(dMinimum * 100.0);
    double dMax = Math.floor(dMaximum * 100.0);

    for (i=0; i<iHeight-50; i++)
    {
      iValue = 768 - (i * MAXCOLORS) / (iHeight-51);

      computeColor(iValue);

      g.setColor(new Color(getRed(), getGreen(), getBlue()));
      g.drawLine(iXBegin+iWidth+50,  iYBegin+iHeight-i,
                 iXBegin+iWidth+100, iYBegin+iHeight-i);
    }

    if (iType == 0)
    {
      g.setColor(Color.black);
      g.drawString("- " + dMin / 100.0,
                   iXBegin+iWidth+120,
                   iYBegin+iHeight+4);
      g.drawString("- " + (dMax - dMin) / 200.0,
                   iXBegin+iWidth+120,
                   iYBegin+iHeight + 4 - (iHeight-50)/2);
      g.drawString("- " + dMax / 100.0,
                   iXBegin+iWidth+120,
                   iYBegin+50 + 4);
    }
    else
    {
      for (i=1; i<iHeight-50; i++)
      {
        iValue = 768 - (i * MAXCOLORS) / (iHeight-51);

        computeColor(iValue);

        if ((iRed   != getRed())   ||
            (iGreen != getGreen()) ||
            (iBlue  != getBlue()))
        {
          iRed   = getRed();
          iGreen = getGreen();
          iBlue  = getBlue();

          g.setColor(Color.black);
          if (iCount < 5)
          {
            g.drawString("- " + dLower[4-iCount],
                         iXBegin+iWidth+120,
                         iYBegin+iHeight-i);

            iCount++;
          }
        }
      }

      g.drawString("- " + dUpper[0], iXBegin+iWidth+120, iYBegin+50 + 4);
    }

    g.setColor(Color.lightGray);
    g.drawRect(iXBegin+iWidth+50,  iYBegin+50, 50, iHeight-50);
  }

  /** Method paint()
   *  <p> This method will paint the Scale data.
   *  @param g - Graphics Handle;
   */

  public void paint(Graphics g)
  {
    drawScale(g);
  }
}
