/*
 * @plotXYDataImage.java Version 1.1 08/26/2015
 *
 * Copyright (c) 2015 Kansas Geological Survey
 * 1930 Constant Avenue, Lawrence, Kansas, 66047, U.S.A.
 * All Rights Reserved.
 */

package plot;

import java.awt.*;
import java.awt.image.*;
import javax.swing.*;

import iqstrat.iqstratHeadersListStruct;
import util.utilLandGridListStruct;      // TRS Land Grid List Structure
import quake.quakeSensorsStruct;         // Sensor Data Structure

import plot.plotXYStruct;
import plot.plotXYData;

/** Class plotXY
 *  <p> This class will create a XY Plot for an array of data
 *
 *  @version 1.1 08/26/2015
 *  @author  John R. Victorine
 */

public class plotXYDataImage extends Canvas
{
  /** Method makePlotImage()
   *  <p> This method will make a image of the Sample Plot
   *  @param iAction = Turn well symbols on or off.
   *  @param stPlot  = XYZ Plot Data Structure
   *  @param stWells = Well Headers List Structure
   */

  public static Image makePlotImage( int                      iAction,
                                     plotXYStruct             stPlot,
                                     iqstratHeadersListStruct stWells )
  {
    Frame frame = new Frame();
    frame.addNotify();

    plotXYData plot = new plotXYData( -1, stPlot, stWells );

    int iWidth  = 900; //50 + plot.getPlotWidth();
    int iHeight = 700; //plot.getPlotHeight();

    plot.setStatus( iAction );

    Image    messageImage = frame.createImage(iWidth, iHeight);
    Graphics graphics     = messageImage.getGraphics();

    graphics.setColor(Color.white);
    graphics.fillRect(0, 0, iWidth, iHeight);

    plot.draw( graphics );

    graphics.dispose();

    return (messageImage);
  }

  /** Method makePlotImage()
   *  <p> This method will make a image of the Sample Plot
   *  @param iAction  = Turn well symbols on or off.
   *  @param iGrid    = Turn TRS Grid on or off.
   *  @param stPlot   = XYZ Plot Data Structure
   *  @param stWells  = Well Headers List Structure
   *  @param stSensor = CO2 Sensor Location Structure
   *  @param stGrid   = TRS Land Grid List Structure
   */

  public static Image makePlotImage( int                      iAction,
                                     int                      iGrid,
                                     plotXYStruct             stPlot,
                                     iqstratHeadersListStruct stWells,
                                     quakeSensorsStruct       stSensor,
                                     utilLandGridListStruct   stGrid )
  {
    Frame frame = new Frame();
    frame.addNotify();

    plotXYData plot = new plotXYData( -1, stPlot, stWells, stSensor, stGrid );

    int iWidth  = 900; //50 + plot.getPlotWidth();
    int iHeight = 700; //plot.getPlotHeight();

    plot.setStatus( iAction );
    plot.setGrid( iGrid );

    Image    messageImage = frame.createImage(iWidth, iHeight);
    Graphics graphics     = messageImage.getGraphics();

    graphics.setColor(Color.white);
    graphics.fillRect(0, 0, iWidth, iHeight);

    plot.draw( graphics );

    graphics.dispose();

    return (messageImage);
  }
}

/*
 *  @version 1.1 08/26/2015
 *  @author  John Victorine
 */