/*
 * @utilLandGridUtility.java Version 1.1 09/27/2012
 *
 * Copyright (c) 2012 Kansas Geological Survey
 * 1930 Constant Avenue, Lawrence, Kansas, 66047, U.S.A.
 * All Rights Reserved.
 */

package util;

import util.utilLandGridListStruct;
import util.utilLandGridStruct;

/** Class utilLandGridUtility
 *  <p> This Class will provide basic utility for the data structures
 *
 *  @version 1.1 09/27/2012
 *  @author  John R. Victorine
 */

public class utilLandGridUtility
{
  /** Method addCounty()
   *  <p> This method will add multiple county grid lines to the Land Grid
   *      List Data Structure.
   *  @param  stOld    = Old Land Grid List Data Structure.
   *  @param  stAdd    = County Grid Lines to be added to Data Structure.
   *  @return stNew    = New Land Grid List Data Structure.
   */

  public static utilLandGridListStruct addCounty( utilLandGridListStruct stOld,
                                                  utilLandGridListStruct stAdd )
  {
    int                    i      = 0;
    int                    k      = 0;
    int                    iRows  = 0;
    utilLandGridListStruct stNew  = new utilLandGridListStruct();

    stNew.iCount = stAdd.iCount;
    if (stOld != null)
      stNew.iCount = stOld.iCount + stAdd.iCount;

    stNew.stItem = new utilLandGridStruct[stNew.iCount];

    if (stOld != null)
    {
      for (i=0; i<stOld.iCount; i++)
      {
		stNew.stItem[iRows] = copy(stOld.stItem[i]);
        iRows++;
      }

      stOld.delete();
      stOld = null;
	}

    for (i=0; i<stAdd.iCount; i++)
    {
 	  stNew.stItem[iRows] = copy(stAdd.stItem[i]);
      iRows++;
    }

    stAdd.delete();
    stAdd = null;

    return (stNew);
  }

  /** Method add()
   * <p> This method will add data structure to the data list structure
   * @param  stNew = new data structure
   * @param  st    = data list structure
   * @return st    = data list structure
   */

  public static utilLandGridListStruct add( utilLandGridStruct stNew,
                                            utilLandGridListStruct st )
  {
    int i      = 0;
    int iCount = 0;
    int iFound = -1;
    utilLandGridListStruct stTemp = new utilLandGridListStruct();

    stTemp.stItem = new utilLandGridStruct[1];
    stTemp.iCount = 0;

    if (st != null)
    {
      stTemp.stItem = new utilLandGridStruct[st.iCount+1];

      for (i=0; i<st.iCount; i++)
      {
        stTemp.stItem[iCount] = copy(st.stItem[i]);
        iCount++;
      }

      st.delete();
      st = null;
    }

    if (iFound == -1)
    {
      stTemp.stItem[iCount] = copy(stNew);
      iCount++;
    }

    st        = new utilLandGridListStruct();
    st.stItem = new utilLandGridStruct[iCount];
    st.iCount = iCount;

    for (i=0; i<iCount; i++)
    {
      st.stItem[i] = copy(stTemp.stItem[i]);
    }

    stTemp.delete();
    stTemp = null;

    return (st);
  }

  /** Method copyList()
   * <p> This method will copy one structure to another
   * @param  stOld = the Old List Structure
   * @return stNew = the New List structure
   */

  public static utilLandGridListStruct copyList(utilLandGridListStruct stOld)
  {
    utilLandGridListStruct stNew = null;

    if (stOld != null)
    {
      stNew        = new utilLandGridListStruct();

      stNew.iCount = stOld.iCount;
      stNew.stItem = new utilLandGridStruct[stOld.iCount];

      for (int i=0; i<stOld.iCount; i++)
      {
        stNew.stItem[i] = copy(stOld.stItem[i]);
      }
    }

    return (stNew);
  }

  /** Method copy()
   * <p> This method will copy one data structure to another
   * @param  stOld = The Old Data Structure
   * @return stNew = The New Data Structure
   */

  public static utilLandGridStruct copy( utilLandGridStruct stOld )
  {
    utilLandGridStruct stNew = null;

    if (stOld != null)
    {
      stNew           = new utilLandGridStruct();

      stNew.sOriginal = new String( stOld.sOriginal );

      stNew.sTownship = new String( stOld.sTownship );
      stNew.sRange    = new String( stOld.sRange );
      stNew.section   = new String( stOld.section );
      stNew.sValue    = new String( stOld.sValue );

      stNew.iTotal = stOld.iTotal;
      if (stNew.iTotal > 0)
      {
        stNew.dLongitude = new double[stNew.iTotal];
        stNew.dLatitude  = new double[stNew.iTotal];

        for (int i=0; i<stNew.iTotal; i++)
        {
		  stNew.dLongitude[i] = stOld.dLongitude[i];
		  stNew.dLatitude[i]  = stOld.dLatitude[i];
		}
      }
    }

    return (stNew);
  }

  /** Method transfer()
   * <p> This method will copy one structure to another
   * @param  stOld = the Old List Structure
   * @return stNew = the New List structure
   */

  public static utilLandGridListStruct transfer(utilLandGridListStruct stOld)
  {
    utilLandGridListStruct stNew = null;

    if (stOld != null)
    {
	  stNew = copyList( stOld );

	  stOld.delete();
	  stOld = null;
	}

    return (stNew);
  }

  /** Method transfer()
   * <p> This method will copy one structure to another
   * @param  stOld = the Old Structure
   * @return stNew = the New structure
   */

  public static utilLandGridStruct transfer(utilLandGridStruct stOld)
  {
    utilLandGridStruct stNew = null;

    if (stOld != null)
    {
	  stNew = copy( stOld );

	  stOld.delete();
	  stOld = null;
	}

    return (stNew);
  }

  /** Method print()
   *  <p> This method will print the Land Grid List Data Structure
   */

  public static void print(utilLandGridListStruct st)
  {
	if (st != null)
	{
	  for (int i=0; i<st.iCount; i++)
	  {
		System.out.println( st.stItem[i].sOriginal +" {"+ st.stItem[i].sTownship +" "+
		                    st.stItem[i].sRange    +" "+ st.stItem[i].section   +"} "+
		                    st.stItem[i].sValue );
		for (int j=0; j<st.stItem[i].iTotal; j++)
		{
		  System.out.println( "-- " + j+" "+ st.stItem[i].dLongitude[j] +" "+ st.stItem[i].dLatitude[j] );
		}
	  }
    }
  }
}